import subprocess

def safe_run_subprocess(command, success_message=None):
    """Safely run any subprocesses and print error messages sensibly"""
    try:
        process = subprocess.Popen(
        command,
        stdout=subprocess.PIPE,  # Capture stdout
        stderr=subprocess.STDOUT,  # Merge stderr into stdout
        text=True,  # Ensure output is in text mode
        )

        # Print output as it's produced
        for line in process.stdout:
            print(line, end='')  # Print each line immediately

        # Wait for process to complete
        return_code = process.wait()

        if return_code != 0:
            raise subprocess.CalledProcessError(return_code, command)

        print(success_message)

    except subprocess.CalledProcessError as e:
        # Surface the error
        print("Error occurred while running the command:")
        print(f"Return code: {e.returncode}")
        print(f"Command: {e.cmd}")
        print("Error output:")
        print(e.output)  # This contains both stdout and stderr

    return return_code


from setuptools import setup, find_packages

setup(
    name="easy_smr",
    description='Easy Sagemaker Ops for R projects',
    long_description="""This package makes it easier to work with Sagemaker by enabling rapid prototyping with local training, processing and deployment.
And correspondingly training, processing and deployment on cloud.
This is very much an experimental package and API is likely to evolve and may break.
Recommended to validate before updating.
    """,
    author='Prateek',
    author_email='prteek@icloud.com',
    version="0.1.11",
    python_requires='>=3.11',
    packages=find_packages(where='.'),
    package_data={
        'easy_smr': [
            'template/easy_smr_base/*.sh',
            'template/easy_smr_base/Dockerfile',
            'template/easy_smr_base/.dockerignore',
            'template/easy_smr_base/training/train',
            'template/easy_smr_base/training/*.R',
            'template/easy_smr_base/processing/.gitkeep',
            'template/easy_smr_base/prediction/*.R',
            'template/easy_smr_base/prediction/serve',
            'template/easy_smr_base/local_test/*.sh',
            'template/easy_smr_base/local_test/test_dir/output/.gitkeep',
            'template/easy_smr_base/local_test/test_dir/model/.gitkeep',
            'template/easy_smr_base/local_test/test_dir/input/data/training/.gitkeep',
            'template/easy_smr_base/renv/.gitignore',
            'template/easy_smr_base/renv/activate.R',
            'template/easy_smr_base/renv/settings.json',
            'template/easy_smr_base/renv.lock',
            'template/easy_smr_base/.Rprofile'
        ]
    },
    install_requires=[
        'click>=8.1.7, <8.1.99',
        'docker>=7.1.0, <7.2.0',
        'sagemaker>=2.226.0, <2.228.0',
    ],
    entry_points={
        'console_scripts': [
            'easy_smr=easy_smr.__main__:cli',
        ],
    }
)


import os
import sys

import boto3
import click
from click import BadParameter
import os
from pathlib import Path
from easy_smr.config.config import ConfigManager
from distutils.dir_util import copy_tree
import re

_FILE_DIR_PATH = os.path.dirname(os.path.realpath(__file__))


def _template_creation(app_name, aws_profile, aws_region, output_dir):
    easy_smr_module_name = 'easy_smr_base'

    easy_smr_exists = os.path.exists(os.path.join(output_dir, easy_smr_module_name))
    if easy_smr_exists:
        raise ValueError(
            "There is a easy_smr directory/module already. "
            "Please, rename it in order to use easy_smr."
        )

    Path(output_dir).mkdir(exist_ok=True)
    Path(os.path.join(output_dir, '__init__.py')).touch()

    # Set 'easy_smr module' directory up
    copy_tree(os.path.join(_FILE_DIR_PATH, '../template'), output_dir)

    # Set configuration file up
    config_manager = ConfigManager(os.path.join(f"{app_name}.json"))
    config = config_manager.get_config()

    config.image_name = app_name
    config.aws_region = aws_region
    config.aws_profile = aws_profile
    config.easy_smr_module_dir = output_dir
    config_manager.set_config(config)


def _get_local_aws_profiles():
    return boto3.Session().available_profiles


def ask_for_app_name():
    app_name = click.prompt(
        text="Type in a name for your SageMaker app (Only alphanumeric characters and - are allowed))",
        type=str
    )

    if not bool(re.fullmatch(r"[a-zA-Z0-9\-]+", app_name)):
        raise BadParameter(
                    message=f"invalid app name: {app_name}. App name must only have alphanumeric characters or dashes '-' "
                    )

    return app_name


def ask_if_existing_project_exists():
    return click.confirm(text="Are you starting a new project?")


def ask_for_root_dir():
    return click.prompt(text="Type in the directory where your code lives. Example: src", type=str).strip('/')


def ask_for_aws_details():
    available_profiles = _get_local_aws_profiles()

    if len(available_profiles) == 0:
        print("aws cli is not configured!")
        return

    valid_positions = list(range(1, len(available_profiles) + 1))
    print("Select AWS profile:")
    print('{}'.format(
            '\n'.join(
                [
                    '{} - {}'.format(pos, profile)
                    for pos, profile in zip(valid_positions, available_profiles)
                ]
            )
        )
    )

    def _validate_profile_position(input_pos):
        if int(input_pos) not in valid_positions:
            raise BadParameter(
                message="invalid choice: {}. (choose from {})".format(
                    input_pos,
                    ', '.join([str(pos) for pos in valid_positions])
                )
            )
        return int(input_pos)

    chosen_profile_index = click.prompt(
        text="Choose from {}".format(', '.join([str(pos) for pos in valid_positions])),
        default=1,
        value_proc=lambda x: _validate_profile_position(x)
    )
    chosen_profile_index -= 1

    chosen_profile = available_profiles[chosen_profile_index]

    chosen_region = click.prompt(
        text="Type in your preferred AWS region name",
        default='eu-west-1',
        type=str
    )

    return chosen_profile, chosen_region


@click.command()
def init():
    """
    Command to initialize SageMaker template
    """

    easy_smr_app_name = ask_for_app_name()

    is_new_project = ask_if_existing_project_exists()

    root_dir = None
    if not is_new_project:
        root_dir = ask_for_root_dir()

    aws_profile, aws_region = ask_for_aws_details()

    _template_creation(
        app_name=easy_smr_app_name,
        aws_profile=aws_profile,
        aws_region=aws_region,
        output_dir=root_dir if root_dir else easy_smr_app_name,
    )

    print("\neasy_smr module is created! ヽ(´▽`)/")



import sys
import click
import os
from easy_smr.commands.helpers import safe_run_subprocess
from easy_smr.config.config import ConfigManager

def _config(app_name):
    config_file_path = os.path.join(f'{app_name}.json')
    if not os.path.isfile(config_file_path):
        raise ValueError("This is not a easy_smr directory: {}".format(os.getcwd()))
    else:
        return ConfigManager(config_file_path).get_config()


def _build(source_dir, image_name, docker_tag):
    """
    Builds a Docker image that contains code under the given source root directory.

    Assumes that Docker is installed and running locally.

    :param source_dir: [str], source root directory
    :param image_name: [str], The name of the Docker image
    :param docker_tag: [str], the Docker tag for the image
    """
    easy_smr_module_path = os.path.relpath(os.path.join(source_dir, 'easy_smr_base/'))

    build_script_path = os.path.join(easy_smr_module_path, 'build.sh')
    dockerfile_path = os.path.join(easy_smr_module_path, 'Dockerfile')

    train_file_path = os.path.join(easy_smr_module_path, 'training', 'train')
    serve_file_path = os.path.join(easy_smr_module_path, 'prediction', 'serve')
    executor_file_path = os.path.join(easy_smr_module_path, 'executor.sh')

    if not os.path.isfile(build_script_path) or not os.path.isfile(train_file_path) or not \
            os.path.isfile(serve_file_path):
        raise ValueError("This is not a easy_smr directory: {}".format(source_dir))

    os.chmod(train_file_path, 0o777)
    os.chmod(serve_file_path, 0o777)
    os.chmod(executor_file_path, 0o777)

    target_dir_name = os.path.basename(os.path.normpath(source_dir))

    command = [
            "{}".format(build_script_path),
            "{}".format(os.path.relpath(source_dir)),
            "{}".format(os.path.relpath(target_dir_name)),
            "{}".format(dockerfile_path),
            docker_tag,
            image_name,
        ]
    safe_run_subprocess(command, "Docker image built successfully!")

@click.command()
@click.option(
    u"-a",
    u"--app-name",
    required=True,
    help="The app name whose json file will be referenced for setting up command"
)
@click.pass_obj
def build(obj, app_name):
    """
    Command to build SageMaker app
    """
    print("Started building SageMaker Docker image. It will take some minutes...\n")

    config = _config(app_name)
    _build(
        source_dir=config.easy_smr_module_dir,
        docker_tag=obj['docker_tag'],
        image_name=config.image_name,
        )
